import * as fs from 'fs';
import { promises } from 'fs';
import * as childProcess from 'child_process';
import * as ariaFs from 'aria-fs';
import * as typescript from 'typescript';
export { typescript as ts };
import * as rollup_plugin_terser from 'rollup-plugin-terser';
import * as _rollup_plugin_commonjs from '@rollup/plugin-commonjs';
import * as _rollup from 'rollup';
import * as magic_string from 'magic-string';
export { default as MagicString } from 'magic-string';
import * as _swc_core from '@swc/core';
import * as esbuild$1 from 'esbuild';
import { EventEmitter } from 'events';
export { default as nodeResolve } from '@rollup/plugin-node-resolve';
export { default as minifyHTML } from 'rollup-plugin-minify-html-literals';

declare const copyFile: typeof promises.copyFile;
declare const writeFile: typeof promises.writeFile;
declare const rename: typeof promises.rename;
declare const readdir: typeof promises.readdir;
declare const readFile: typeof promises.readFile;
declare const stat: typeof promises.stat;
declare const unlink: typeof promises.unlink;
declare const symlink: typeof promises.symlink;
declare const readlink: typeof promises.readlink;
declare const mkdir: typeof promises.mkdir;
declare const lstat: typeof promises.lstat;

declare const nodeMajorVersion: number;
declare const exist: typeof fs.exists.__promisify__;
declare const exec: typeof childProcess.exec.__promisify__;

declare const globFiles: typeof ariaFs.globFiles;
declare const mkdirp: typeof ariaFs.mkdirp;
declare const clean: typeof ariaFs.clean;
declare const copyFiles: typeof ariaFs.copyFiles;
declare const symlinkDir: (src: string, dest: string) => Promise<void>;
declare const unlinkDir: (dest: string) => Promise<void>;
declare const unlinkFile: (dest: string) => Promise<void>;
declare const symlinkFile: (src: string, dest: string) => Promise<void>;

declare const replacePlugin: any;
declare type WatcherOptions = _rollup.WatcherOptions;
declare type ModuleFormat = _rollup.ModuleFormat;
declare const rollup: typeof _rollup.rollup;
declare const watch: typeof _rollup.watch;

declare function commonjs(options?: _rollup_plugin_commonjs.RollupCommonJSOptions): any;
declare function terser(options?: rollup_plugin_terser.Options): any;

interface PackageFile {
    filePath?: string;
    entry?: string;
    output?: string;
    format?: string;
    name?: string;
    main?: string;
    module?: string;
    typings?: string;
    dependencies?: KeyValue;
    devDependencies?: KeyValue;
    peerDependencies?: KeyValue;
}
interface KeyValue {
    [key: string]: string;
}
declare type PluginOptions = any[] | PluginBeforeAfter;
interface PluginBeforeAfter {
    before?: any[];
    after?: any[];
}
interface CreateRollupConfigOptions {
    config: RollupConfigBase | TSRollupConfig | RollupConfigBase[] | TSRollupConfig[];
    name?: string;
    esbuild?: boolean;
    swc?: boolean;
    write?: boolean;
}
interface OutputOptions extends RollupConfigOutput {
}
interface RollupConfigBase {
    input?: string | string[];
    external?: string[];
    plugins?: PluginOptions;
    output?: RollupConfigOutput | RollupConfigOutput[];
    resolveOpts?: NodeResolveOptions;
    commonOpts?: CommonJsOptions;
    replace?: KeyValue;
    compress?: boolean;
    watch?: WatcherOptions;
    hmr?: boolean;
}
interface TSRollupConfig extends RollupConfigBase {
    tsconfig?: TSConfigOptions;
}
interface NodeResolveOptions {
    extensions?: string[];
    mainFields?: string[];
}
interface CommonJsOptions {
    extensions?: string[];
    include?: string | string[];
    exclude?: string | string[];
}
interface RollupConfigOutput {
    sourcemap?: boolean | 'inline' | 'hidden';
    file?: string;
    format?: ModuleFormat;
    name?: string;
    exports?: string;
    globals?: KeyValue;
    plugins?: PluginOptions;
}
interface TSConfigOptions {
    compilerOptions?: typescript.CompilerOptions;
    transformers?: typescript.CustomTransformers;
    exclude?: string[];
    include?: string[];
}
interface CreateTSConfigOptions {
    input?: string | string[];
    file?: string;
    tsconfig?: TSConfigOptions;
    pluginOpts?: any;
}
interface TSRollupPluginResult {
    tsconfigDefaults: {
        compilerOptions?: Omit<typescript.CompilerOptions, 'module' | 'moduleResolution' | 'target'>;
        exclude?: string[];
        include?: string[];
    };
    transformers?: any[];
    check?: boolean;
    objectHashIgnoreUnknownHack?: boolean;
    useTsconfigDeclarationDir?: boolean;
    cacheRoot?: string;
}
declare const defineConfig: (config: Pick<TSRollupConfig, 'plugins' | 'external' | 'output'>) => Pick<TSRollupConfig, "output" | "plugins" | "external">;

declare const DEFAULT_DEST = "dist";
declare const DEFAULT_SOURCE = "src";
declare const DEFAULT_VALUES: Readonly<{
    DIST_FOLDER: string;
    SOURCE_FOLDER: string;
    ROLLUP_EXTERNALS: string[];
}>;
declare function baseDir(): string;
declare function getInputEntryFile(input: string): string;
declare function getPackageNameSync(filePath?: string): any;
declare function onwarn(options: {
    code: string;
    message: string;
}): void;

declare function copyPackageFile(options?: PackageFile): Promise<void>;

interface CopyReadmeOptions {
    filePath?: string;
    output?: string;
}
declare function copyReadMeFile(options?: CopyReadmeOptions): Promise<void>;

declare function findTargetBuild(target: string, config: RollupConfigBase[] | TSRollupConfig[]): Promise<void>;

declare function getPackageNameScope(name: string): string;
declare function getPackage(filePath?: string): Promise<any>;
declare function getPackageName(filePath?: string): Promise<string>;

declare const DEFAULT_OUT_DIR = "dist";
declare type OutputFormat = 'es' | 'cjs' | 'umd' | 'iife';
interface BuildOptions {
    entry?: string;
    d?: boolean;
    declaration?: boolean;
    format?: string;
    external?: string;
    name?: string;
    globals?: string;
    clean?: string;
    sourcemap?: boolean | 'inline' | 'hidden';
    config?: string;
    output?: string;
    compress?: boolean | string;
    resolve?: boolean | string;
    watch?: boolean;
    target?: string;
    expirement?: boolean;
    bundler?: 'esbuild' | 'swc' | 'ts';
    esbuild?: boolean;
    swc?: boolean;
    write?: boolean;
    dtsOnly?: boolean;
}
interface TestAriaConfigOptions extends Omit<AriaConfigOptions, 'test'> {
    scripts?: string[];
}
interface AriaConfigOptions {
    external?: string[];
    plugins?: PluginOptions;
    output?: {
        globals?: KeyValue;
    };
    test?: TestAriaConfigOptions;
}
interface BuildFormatOptions extends BuildOptions {
    pkgName?: string;
    plugins?: PluginOptions;
    dependencies?: string[];
    dtsOnly?: boolean;
}

declare function getAriaConfig(config?: string): Promise<AriaConfigOptions>;

declare function getEntryFile(pkgName: string): string;

declare function bundlerOptions(options?: Pick<BuildOptions, 'swc' | 'esbuild'>): {
    swc: boolean;
    esbuild: boolean;
};
declare function handler(options: BuildOptions): Promise<void>;

declare function run(version: string): Promise<void>;

interface ExternalDepsOptions {
    external?: string;
    dependencies?: string[];
}
declare function getExternalDeps(options: ExternalDepsOptions): string[];
declare function entryFile(formats: string[] | string, entry: string, module?: string): string;
declare function updateExternalWithResolve(options?: {
    resolve?: boolean | string;
    external?: string[];
}): string[];
declare function parseConfig(options?: {
    config?: string;
    entry?: string;
}): string;
declare function getPkgDependencies(pkgJson: PackageFile): string[];
declare function mergeGlobals(globals?: KeyValue, optionGlobals?: string): string;
declare function mergeExternal(externals: string, external: string[]): string;
declare function parsePlugins(plugins?: PluginOptions): any[] | {
    before: any[];
    after: any[];
};

declare const getCliOptions: () => {
    package: string;
    command: string;
    options: ({
        alias: string;
        description: string;
        defaultValue?: undefined;
    } | {
        alias: string;
        description: string;
        defaultValue: boolean;
    } | {
        alias: string;
        description: string;
        defaultValue: string;
    })[];
};

interface GetFileOptions {
    outDir: string;
    name: string;
    formats: string[];
    format?: ModuleFormat;
    entry?: string;
}
declare function buildConfig(options: BuildFormatOptions): TSRollupConfig;

declare function isGlob(file: string): boolean;
interface TargetCopyOptions {
    src: string;
    dest: string;
    recursive?: boolean;
    replace?: (filename: string) => Promise<void>;
}
interface RollupPluginCopyOptions {
    hook?: string;
    targets?: TargetCopyOptions[];
    copyEnd?: () => Promise<void>;
}
declare function replaceContent(options?: {
    filename?: string;
    strToFind?: string;
    strToReplace?: string;
    extensions?: string[];
}): Promise<void>;
declare function createOutfile(file: string, dest: string, recursive: boolean): string;
declare function copyAssets(options?: RollupPluginCopyOptions): Promise<void>;

declare function copy(options?: RollupPluginCopyOptions): {
    [x: string]: string | (() => Promise<void>);
    name: string;
};

interface LinkOptions {
    hook?: string;
    moduleDir?: string;
    outDir?: string;
    targets?: {
        package?: string;
        dest?: string;
    }[];
}
declare function linkToPackages(options: LinkOptions): {
    [x: string]: string | (() => Promise<void>);
    name: string;
};

declare function transform(file: string, content: string, specifiers: KeyValue): {
    code: string;
    map: magic_string.SourceMap;
};
declare function transformImport(specifiers?: KeyValue): {
    name: string;
    transform(code: string, id: string): {
        code: string;
        map: magic_string.SourceMap;
    };
};

declare function swcPlugin(options?: _swc_core.JscConfig): {
    name: string;
    buildStart: () => Promise<void>;
    transform(code: string, id: string): Promise<_swc_core.Output>;
};

declare function pathResolver(extensions?: string[]): (id: string, origin: string | undefined) => string;
declare function resolvePathPlugin(extenstions?: string[]): {
    name: string;
    resolveId: (id: string, origin: string) => string;
};

interface EsBuildPluginOptions {
    transformOptions?: esbuild$1.TransformOptions;
    extensions?: string[];
}
declare function transformCode(options?: esbuild$1.TransformOptions): (code: string, id: string) => Promise<{
    code: string;
    map: string;
}>;
declare function esBuildPlugin(options?: EsBuildPluginOptions): {
    name: string;
    transform(code: string, id: string): Promise<{
        code: string;
        map: string;
    }>;
};

declare function esbuild(options: CreateRollupConfigOptions): Promise<_rollup.RollupOutput[][]>;

interface CreateRollupConfigBuilderOptions extends CreateRollupConfigOptions {
    pkg?: PackageFile;
    dtsOnly?: boolean;
}
declare function createOptions(options: CreateRollupConfigBuilderOptions | BuildFormatOptions): Promise<CreateRollupConfigBuilderOptions>;
declare function bundle(options: CreateRollupConfigBuilderOptions | BuildFormatOptions): Promise<void>;

declare function esbuildDts(options: CreateRollupConfigBuilderOptions): Promise<_rollup.RollupOutput[]>;

interface Listener {
    (file?: string, stats?: fs.Stats): void;
}
interface ReadyListener {
    (files: string[]): void;
}
interface WatchOptions {
    onChange?: Listener;
    onCreate?: Listener;
    onDelete?: Listener;
    onReady?: ReadyListener;
}
declare class Watcher extends EventEmitter {
    private watchers;
    private store;
    constructor(options?: WatchOptions);
    get watchFiles(): any[];
    watch(src: string): Promise<void>;
    unwatch(src: string): Promise<void>;
}
declare const watcher: (src: string, options?: WatchOptions) => Promise<void>;

export { AriaConfigOptions, BuildFormatOptions, BuildOptions, CommonJsOptions, CopyReadmeOptions, CreateRollupConfigBuilderOptions, CreateRollupConfigOptions, CreateTSConfigOptions, DEFAULT_DEST, DEFAULT_OUT_DIR, DEFAULT_SOURCE, DEFAULT_VALUES, EsBuildPluginOptions, ExternalDepsOptions, GetFileOptions, KeyValue, LinkOptions, Listener, ModuleFormat, NodeResolveOptions, OutputFormat, OutputOptions, PackageFile, PluginBeforeAfter, PluginOptions, ReadyListener, RollupConfigBase, RollupConfigOutput, RollupPluginCopyOptions, TSConfigOptions, TSRollupConfig, TSRollupPluginResult, TargetCopyOptions, TestAriaConfigOptions, WatchOptions, Watcher, WatcherOptions, baseDir, buildConfig, bundle, bundlerOptions, clean, commonjs, copy, copyAssets, copyFile, copyFiles, copyPackageFile, copyReadMeFile, createOptions, createOutfile, defineConfig, entryFile, esBuildPlugin, esbuild, esbuildDts, exec, exist, findTargetBuild, getAriaConfig, getCliOptions, getEntryFile, getExternalDeps, getInputEntryFile, getPackage, getPackageName, getPackageNameScope, getPackageNameSync, getPkgDependencies, globFiles, handler, isGlob, linkToPackages, lstat, mergeExternal, mergeGlobals, mkdir, mkdirp, nodeMajorVersion, onwarn, parseConfig, parsePlugins, pathResolver, readFile, readdir, readlink, rename, replaceContent, replacePlugin, resolvePathPlugin, rollup, run, stat as stats, swcPlugin, symlink, symlinkDir, symlinkFile, terser, transform, transformCode, transformImport, unlink, unlinkDir, unlinkFile, updateExternalWithResolve, watch, watcher, writeFile };
